#!/usr/local/bin/perl

###
###  Version 1.1
###
###  Written by Ron Tapia. Based on Formmail written by Matt Wright.
###  Modified, maintained, and released under GPL by Aaron Hachez. 
###  (aaronh@hollowworks.com)
###  
###  Copyright 2000, Ron Tapia
###
###  This program is free software; you can redistribute it and/or modify
###  it under the terms of the GNU General Public License as published by
###  the Free Software Foundation; either version 2 of the License, or
###  (at your option) any later version.
###
###  This program is distributed in the hope that it will be useful,
###  but WITHOUT ANY WARRANTY; without even the implied warranty of
###  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
###  GNU General Public License for more details.
###
###  You should have received a copy of the GNU General Public License
###  along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###

use CGI;

###
###  User config
###

###
###  DATA_FILENAME      - the name of a file where we write the data in 
###                       DATA_DELIMITER separated form.
###
###  DATA_DELIMITER     - the data delimiter for DATA_FILENAME
###
###  TEMPLATE_DIRECTORY - the directory where we look for response page
###                       templates
###
###  SENDMAIL_COMMAND   - the command used to send email
###
###  PASSWORD           - the password -- CHANGE PW BELOW IF YOU USE THIS FEATURE
###
###  @valid_email       - list of strings to match against the recipient list to
###                       prevent email sent to recipients other than the owner.
###                       NOTE: For all entries you should use the form \@domain.com
###                       or full.user\@domain.com -- If you omit the \@ in the first
###                       case, it is possible that the form could be more likely
###                       abused.
###
###  Note: The following feature depreciated in favor of @valid_email
###  If you wish to enable referer checking, uncomment all sections begining with 
###  the remark "# REFERER-UN"
###
###  @referers          - list of strings to match the referring web site against
###                       error on failure, this helps keep people from using the
###                       script to spam or scam. This feature does not provide
###                       the same security as email validation (@valid_email)
###                       and should only be used in conjunction with, not in
###                       place of.
###

$DATA_FILEMAPFILENAME = "/full/system/path/to/.foomail/filemap";
$DATA_DELIMITER       = ",";
$TEMPLATE_DIRECTORY   = "/full/system/path/to/.foomail/templates";
$SENDMAIL_COMMAND     = "/usr/sbin/sendmail -t";
$PASSWORD             = "snoopy";
@valid_email          = ("\@vaild-domain.com", "single\@email-address.com");

# REFERER-UN
# This option depreciated in favor of email validation (Referers may be spoofed)
#@referers             = ("yourdomain.com", "some-otherdomain.com");
###

###
### End User Config
###


###
### currently used templates filenames:
###
###           error.html        - for displaying errors
###           bad_password.html - displayed when the password is invalid
###           success.html      - displayed on success
###
###

my($q) = CGI->new();

my(@names)  = $q->param();

my(@dont_send_fields) = (
			 "recipient",
			 "subject",
			 "output",
			 "redirect"
			 );

###
### get references to lists of values, remove them from the parameter list
###
$required     = get_pparam($q, "required");
$env_report   = get_pparam($q, "env_report");
$print_config = get_pparam($q, "print_config");
$recipients   = get_pparam($q, "recipient");
$write_fields = get_pparam($q, "write_fields");
$email_fields = get_pparam($q, "email_fields");

###
### get some single values, remove them from the parameter list
###
$missing_fields_redirect = get_param($q, "missing_fields_redirect");
$redirect                = get_param($q, "redirect");

###
### Check refering URL (Depreciated in favor of recipient validation)
###
# REFERER-UN
# my($refr) = $ENV{"HTTP_REFERER"};
# if (@referers && (ok_referer($refr) != 1)) {
#     send_error("Bad referer: $refr");
#     exit(1);
# }
###

###
### If email does not match a valid email address or domain, send error and exit.
###
my($bad_email);
if ($bad_email = ok_email($recipients)) {
    send_error("Error: $bad_email, is not a valid recipient.");
    exit(1);
}

###
### If one or more parameters if missing, send an error page (or redirect) and exit.
###
my(@missing_params);
if(@missing_params = check_required($q, $required)) {
    if($missing_fields_redirect) {
	send_redirect($missing_fields_redirect);
    } else {
	send_error("Some required parameters are missing: " . join(",", @missing_params));
    }
    exit(1);
}

###
### If the password doesn't match, send a page.
###
if (!match_password($q)) {
    send_bad_password($q);
    exit(1);
}

###
### Write the data file.
###
if (!write_data_file($q, $write_fields)) {
    send_error("Couldn't write data file.\n");
    exit(1);
}

###
### Send email.
###
if (!send_email($q, $recipients, $env_report, $email_fields)) {
    send_error("Couldn't send email.\n");
    exit(1);
}


###
### Send an html response.
###
if ($redirect) {
    send_redirect($redirect);
} else {
    send_success_page($q, $recipients, $email_fields);
}

exit 0;

########################################################
########################################################
########################################################

# REFERER-UN
# sub ok_referer {
#     my($r) = $_[0];
#     my($i);
#     my($domain);
# 
#     if ($refr =~ /http:\/\/([^\/]+)/) {
# 	$domain = $1;
#     } else {
# 	return 0;
#     }
# 
#     for $i (@referers) {
#     if ($domain =~ /$i$/) {
#         return 1;
#     }
#     }
#     return 0;
# }

sub ok_email {
    my($e) = $_[0];
    my($i);
    my($j);
    my($match) = 0;

    for $i (@$e) {
	    for $j (@valid_email) {
            if ($i =~ /$j$/) {
                $match = 1;
                next;
            }
        }

        if (!$match) {
            return $i;
        }

        $match = 0;
    }

    return 0;
}

sub send_redirect {
    my($redirect_url) = $_[0];
    printf "Location: %s\n\n", $redirect_url;
}

sub match_password {
    my($q) = $_[0];

    my($pass) = get_param($q,"password");
    return ($pass eq $PASSWORD);
}

sub send_success_page {
    my($q) = $_[0];
    my($r) = $_[1];
    my($w) = $_[2];

    print "Content-Type: text/html\n\n";

    open(SUCCESS_PAGE, $TEMPLATE_DIRECTORY . "/success.html") || die "Couldn't open response template.";

    my($s);
    for $i (@$w) {
    $s .= sprintf("<b>%s:</b> %s<br>\n", $i, $q->param($i));
    }
    
    my($recipient_string) = join(", ", @{$r});

    while(<SUCCESS_PAGE>) {
	s/MY_VARIABLE_TABLE/$s/g;
	s/MY_RECIPIENTS/$recipient_string/g;
	print;
    }

}

sub check_required {
    my($q)        = $_[0];
    my($required) = $_[1];
    
    my(@fields)   = split(/,/, get_param($q, "required"));
    my($valid_email);
    my($i) = 0;
    my(@missing);
    foreach $blah (@fields) {
	$blah =~ s/ //;
	$newblah = get_param($q, $blah);
	if (!$newblah) {
	    @missing[$i] = $blah;
	    ++$i;
	}
	if ($blah eq "email") {
	    $valid_email = split(/\@/,$newblah);
	    if (!($valid_email eq 2)) {
		@missing[$i] = $blah;
		++$i;
	    }
	}
    }

    return (@missing);
}
 

##
## plural parameters
##
sub get_pparam {
    my($q) = $_[0];
    my($p) = $_[1];

    my($return) = [split(/,/, $q->param($p))];

    return $return;
}

sub get_param {
    my($q) = $_[0];
    my($p) = $_[1];
    my($return) = $q->param($p);
    return $return;
}

sub send_email {
    my($q) = $_[0];
    my($r) = $_[1];
    my($e) = $_[2];
    my($w) = $_[3];
    
    open(SENDMAIL,"|" . $SENDMAIL_COMMAND) || return 0;
    my($subject) = get_param($q, "subject");

    my(@recipients) = ();

    for $i (@$r) {
	push (@recipients, $i);
    }

    my($to)      = shift(@recipients);

    printf SENDMAIL "To: %s\n", $to;
    if (@{$r}) {
	printf SENDMAIL "Cc: %s\n", join(",", @recipients);
    }
    printf SENDMAIL "Subject: %s\n", $subject;
    print  SENDMAIL "\n";

    my($i);
    print SENDMAIL "Parameters:\n\n";
    for $i (@{$w}) {
	printf SENDMAIL "\t%s: %s\n", $i, $q->param($i);
    }

    if(@{$e}) {
	print SENDMAIL "\n";
	print SENDMAIL "Environment Variables:\n\n";
	for $i (@{$e}) {
	    printf SENDMAIL "\t%s: %s\n", $i, $ENV{$i};
	}
    }
    print SENDMAIL "\n";

    return 1;
}

sub write_data_file {
    my($q) = $_[0];
    my($w) = $_[1];

    my($f) = get_param($q, "output");

    if(!$f) {
	return 0;
    }

    open(MAP_FILE, $DATA_FILEMAPFILENAME) || die "Couldn't open map file!";

    if (! lock_file(\*MAP_FILE)) {
	return 0;
    }

    my($matched) = 0;
    while(<MAP_FILE>) {
	chmop;
	($spec, $filename) = split(/\s+/);
	if ($spec eq $f) {
	    $matched = 1;
	    last;
	}
    }
    
    close(MAP_FILE);
    
    if ((!$matched) || (!$filename)) {
	return 0;
    }

    open(DATA_FILE, ">>" . $filename) || die "Couldn't write data file!";

    my($i);
    my($delim) = "";
    for $i (@{$w}) {
	print DATA_FILE $delim;
	print DATA_FILE $q->param($i);
	$delim = $DATA_DELIMITER;
    }
    print DATA_FILE "\n";

    return 1;
}

sub send_bad_password {
    my($q) = $_[0];

    open(BAD_PASSWORD_PAGE, $TEMPLATE_DIRECTORY . "/bad_password.html") || die "Couldn't open bad_passwd template!";

    print "Content-Type: text/html\n\n";

    while(<BAD_PASSWORD_PAGE>) {
	print;
    }

}

sub send_error {
    my($error_string) = $_[0];

    print "Content-Type: text/html\n\n";

    #not much point in checking for an error.
    open(ERROR_PAGE, $TEMPLATE_DIRECTORY . "/error.html") || die "Couldn't open error template!";

    while(<ERROR_PAGE>) {
	s/MY_ERROR_MESSAGE/$error_string/g;
	print;
    }
}

sub lock_file {
    my($fh)   = shift;

    my($LOCK_TRIES)      = 30;
    my($LOCK_SLEEP_TIME) = 5;

    # Mnemonic for locking.
    my($LOCK_SH) = 1;
    my($LOCK_EX) = 2;
    my($LOCK_NB) = 4;
    my($LOCK_UN) = 8;

    my($i)      = 0;
    my($locked) = 0;

    while ($i < $LOCK_TRIES) {
        if (flock($fh, $LOCK_EX)) {
            $locked = 1;
            last;
        }
        sleep($LOCK_SLEEP_TIME);
        $i++;
    }

    if ($locked) {
        return 1;
    } else {
        return 0;
    }
}








